* *********************************************************************
*     
*
*               FORTRAN Package for UNPOLARIZED FRAGMENTATION FUNCTIONS
*
*    (Determination of pion and kaon fragmentation functions including spin asymmetries
*                     data in a global analysis, arXiv:1306.1612v1)                 
*            
*            M. Soleymaninia, A. Khorramian, M. Moosavinejad, F. Arbabifar          
*                                                                  
*      							  Aug,05,2013

*             Subroutine SKMA(Q2,X,IO,IC,FF,GRAD):
*
*  
*   INPUT:
*      
*     IO=  1: Pion LO 
*          2: Pion NLO
*          3: Kaon LO
*          4: Kaon NLO
*         
*
*     IC=1: pi^+, K^+
*     IC=2: pi^-, K^-
*     IC=3: pi^0=[pi^+ + pi^-]/2, [K^0+K^0b]/2
*
*
*            X                    (between  0.01   and  1.0)    
*            Q2 = scale in GeV**2 (between  1.0    and  1.D8)
*
*  
*
*   OUTPUT: Fragmentation Functions FF(-5:5) 
*
*     FF(I) :
*      I =  1 ... u 
*           2 ... d  
*           3 ... s 
*           4 ... c  
*           5 ... b 
*           0 ... gluon  
*          -5 ... bbar 
*          -4 ... cbar 
*          -3 ... sbar 
*          -2 ... dbar 
*          -1 ... ubar 
*           
*           
*
*
*      FFs values are not multiplied by x.
*
c *********************************************************************
      SUBROUTINE SKMA(Q2,X,IO,IC,FF)
c ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NQ=33, NX=38, ND=13)
      DIMENSION IREAD(2), QG(NQ), XG(NX), PARTJ1(ND), PARTJ2(ND)
     .         ,FF(-5:5),BXG(NX,NQ,ND), CXG(NX,NQ,ND), DXG(NX,NQ,ND)
     .         ,PARTG(NX,NQ,ND)

     
      SAVE IREAD, BXG, CXG, DXG, PARTG, QG, XG 
      DATA IREAD /1, 1/ 

      mc2=2.0449D0
      mb2=18.490D0
      EPS=1.D-12



c---CALCULATE SPLINE COEFFICIENTS---.
      IF((IREAD(1).NE.1).AND.(IREAD(2).EQ.IO)) GO TO 20
      mc2=mc2-EPS
      mb2=mb2-EPS

c---READ GRID DATA AND CALCULATE SPLINE COEFFICIENTS---. 
      IF(IO.EQ.1) THEN
        OPEN(20,FILE='pilo.grid')

      ELSE IF(IO.EQ.2) THEN
        OPEN(20,FILE='pinlo.grid')

      ELSE IF(IO.EQ.3) THEN
        OPEN(20,FILE='kalo.grid')

      ELSE IF(IO.EQ.4) THEN
        OPEN(20,FILE='kanlo.grid')

      ELSE
        WRITE(*,100) IO
 100   FORMAT(' ','SKMA ERROR: IO =', I3)
        STOP
      END IF


      DO J=1,NQ
        DO K=1,NX
          READ(20,1025) XG(K),QG(J),
     .                  PARTG(K,J,1), PARTG(K,J,2), PARTG(K,J,3),
     .                  PARTG(K,J,4), PARTG(K,J,5), PARTG(K,J,6), 
     .                  PARTG(K,J,7), PARTG(K,J,8)

        ENDDO
      ENDDO

 1025 FORMAT (10(1PE10.3))
      CLOSE(20)

c---x=1 FF=0.D0---
      DO I=1,ND
        DO J=1,NQ
          PARTG(NX,J,I)=0.D0 
          CALL LSPLINE(NX,XG,PARTG,BXG,CXG,DXG,IO,I,J)
        ENDDO
      ENDDO

      IREAD(1)=2
      IREAD(2)=IO
   20 CONTINUE

      DO I=1,11
        FF(I-6)=0.D0
      END DO

c---CHECK X AND Q2 VALUES---
      IF(IC.GT.3) THEN
        WRITE(*,120) IC
 120   FORMAT(' ','SKMA ERROR: IC =', I3)
        STOP
      ENDIF

      IF((X.LT.5.D-2).OR.(X.GT.1.D0)) THEN
        WRITE(*,130) X
 130   FORMAT (' ','FF WARNING: OUT OF RANGE --> X =', 1PE12.3)
        STOP
      ENDIF
      IF((Q2.LT.1.D0).OR.(Q2.GT.1.D8)) THEN
        WRITE(*,140) Q2
 140   FORMAT (' ','FF WARNING: OUT OF RANGE --> Q2 =', 1PE12.3)
        STOP
      ENDIF

c---INTERPOLATION---.
c---X: CUBIC SPLINE INTERPOLATION, LOG(Q2): LINEAR INTERPOLATION---.
      J=ISERCH(NQ,QG,Q2)
      IF(J.EQ.NQ) J=NQ-1
      K=ISERCH(NX,XG,X)
      DO I=1,ND
        DX=X-XG(K)
        PARTJ1(I)=PARTG(K,J,I)
     >       +DX*(BXG(K,J,I)+DX*(CXG(K,J,I)+DX*DXG(K,J,I)))
        PARTJ2(I)=PARTG(K,J+1,I)
     >       +DX*(BXG(K,J+1,I)+DX*(CXG(K,J+1,I)+DX*DXG(K,J+1,I)))
      ENDDO

c--- Fragmentation functions---
      T=(DLOG(Q2)-DLOG(QG(J)))/(DLOG(QG(J+1))-DLOG(QG(J)))
      DO I=1,3
        FF(I-1)=(1.D0-T)*PARTJ1(I)+T*PARTJ2(I)     ! g, u, d
        FF(-I)=(1.D0-T)*PARTJ1(I+3)+T*PARTJ2(I+3)  ! ub, [db, s]^pi+, [sb, s]^K; 
        FF(I+2)=(1.D0-T)*PARTJ1(I+5)+T*PARTJ2(I+5) ! s, c, b
      ENDDO
      IF(Q2.LT.mc2) FF(4)=0.D0
      IF(Q2.LT.mb2) FF(5)=0.D0
      FF(-4)=FF(4) ! cb=c
      FF(-5)=FF(5) ! bb=b

      IF((IO.EQ.3).or.(IO.EQ.4)) THEN  ! K+ 
        FF(-3)=FF(-2) ! FF(-2)=sb -> FF(-3)
        FF(-2)=FF(2)  ! db=d
      ENDIF

      IF(IC.EQ.2) THEN ! Negative charge
        IF(IO.LT.3) THEN  ! pi:u<->ub, d<->db
          TEMP1=FF(-1);TEMP2=FF(2)
          FF(-1)=FF(1);FF(2)=FF(-2)
          FF(1)=TEMP1;FF(-2)=TEMP2

        ELSE IF((IO.EQ.3).OR.(IO.EQ.4)) THEN ! K:u<->ub, s<->sb
          TEMP1=FF(-1);TEMP2=FF(3)
          FF(-1)=FF(1);FF(3)=FF(-3)
          FF(1)=TEMP1;FF(-3)=TEMP2

        ENDIF

      ELSE IF(IC.EQ.3) THEN ! (pi^+ + pi^-)/2, (K^0 + K^0b)/2, (p+pb)/2
        IF((IO.EQ.3).OR.(IO.EQ.4)) THEN ! K^0: u<->d, ub<->db
          TEMP1=FF(-1);TEMP2=FF(1)
          FF(-1)=FF(-2);FF(1)=FF(2)
          FF(-2)=TEMP1;FF(2)=TEMP2
        ENDIF
        DO I=1,5 ! (pi^+ + pi^-)/2, (K^0 + K^0b)/2, (p+pb)/2
          FF(I)=(FF(I)+FF(-I))/2.D0
          FF(-I)=FF(I)
        ENDDO
      ENDIF


      RETURN
      END
c ---------------------------------------------------------------------
      SUBROUTINE LSPLINE(N,X,Y,B,C,D,IO,I,J)
c ---------------------------------------------------------------------
c CALCULATE THE COEFFICIENTS B,C,D IN A CUBIC SPLINE INTERPOLATION.
c INTERPOLATION SUBROUTINES ARE TAKEN FROM
c G.E. FORSYTHE, M.A. MALCOLM AND C.B. MOLER,
c COMPUTER METHODS FOR MATHEMATICAL COMPUTATIONS (PRENTICE-HALL, 1977).
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NQ=33, NX=38, ND=13)
      DIMENSION Y(NX,NQ,ND),B(NX,NQ,ND),C(NX,NQ,ND),D(NX,NQ,ND)
     1         ,X(NX) 
      NM1=N-1
      IF(N.LT.2) RETURN
      IF(N.LT.3) GO TO 250
      D(1,J,I)=X(2)-X(1)
      C(2,J,I)=(Y(2,J,I)-Y(1,J,I))/D(1,J,I)
      DO 210 K=2,NM1
        D(K,J,I)=X(K+1)-X(K)
        B(K,J,I)=2.0D0*(D(K-1,J,I)+D(K,J,I))
        C(K+1,J,I)=(Y(K+1,J,I)-Y(K,J,I))/D(K,J,I)
        C(K,J,I)=C(K+1,J,I)-C(K,J,I)
  210 CONTINUE
      B(1,J,I)=-D(1,J,I)
      B(N,J,I)=-D(N-1,J,I)
      C(1,J,I)=0.0D0
      C(N,J,I)=0.0D0
      IF(N.EQ.3) GO TO 215
      C(1,J,I)=C(3,J,I)/(X(4)-X(2))-C(2,J,I)/(X(3)-X(1))
      C(N,J,I)=C(N-1,J,I)/(X(N)-X(N-2))-C(N-2,J,I)/(X(N-1)-X(N-3))
      C(1,J,I)=C(1,J,I)*D(1,J,I)**2.0D0/(X(4)-X(1))
      C(N,J,I)=-C(N,J,I)*D(N-1,J,I)**2.0D0/(X(N)-X(N-3))
  215 CONTINUE
      DO 220 K=2,N
        T=D(K-1,J,I)/B(K-1,J,I)
        B(K,J,I)=B(K,J,I)-T*D(K-1,J,I)
        C(K,J,I)=C(K,J,I)-T*C(K-1,J,I)
  220 CONTINUE
      C(N,J,I)=C(N,J,I)/B(N,J,I)
      DO 230 IB=1,NM1
        K=N-IB
        C(K,J,I)=(C(K,J,I)-D(K,J,I)*C(K+1,J,I))/B(K,J,I)
  230 CONTINUE
      B(N,J,I)=(Y(N,J,I)-Y(NM1,J,I))/D(NM1,J,I)
     1        +D(NM1,J,I)*(C(NM1,J,I)+2.0D0*C(N,J,I))
      DO 240 K=1,NM1
        B(K,J,I)=(Y(K+1,J,I)-Y(K,J,I))/D(K,J,I)
     1          -D(K,J,I)*(C(K+1,J,I)+2.0D0*C(K,J,I))
        D(K,J,I)=(C(K+1,J,I)-C(K,J,I))/D(K,J,I)
        C(K,J,I)=3.0D0*C(K,J,I)
  240 CONTINUE
      C(N,J,I)=3.0D0*C(N,J,I)
      D(N,J,I)=D(N-1,J,I)
      RETURN
  250 CONTINUE
      B(1,J,I)=(Y(2,J,I)-Y(1,J,I))/(X(2)-X(1))
      C(1,J,I)=0.0D0
      D(1,J,I)=0.0D0
      B(2,J,I)=B(1,J,I)
      C(2,J,I)=0.0D0
      D(2,J,I)=0.0D0
      RETURN
      END
c ---------------------------------------------------------------------
      INTEGER FUNCTION ISERCH(N,X,Y)
c ---------------------------------------------------------------------
c THIS FUNCTION SEARCHES "I" WHICH SATISFIES THE RELATION
c X(I) <= Y < X(I+1) BY USING A BINARY SEARCH.
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(38)

      MIN=1
      MAX=N+1

   10 CONTINUE
      MID=(MIN+MAX)/2
      IF(Y.LT.X(MID)) THEN
        MAX=MID
      ELSE
        MIN=MID
      END IF
      IF((MAX-MIN).GT.1) GO TO 10

      ISERCH=MIN

      RETURN
      END
c *********************************************************************
